/* eslint-disable no-magic-numbers */
import { openReactionDrawerById, selectTab } from '@/redux/drawer/drawer.slice';
import { PluginsEventBus } from '@/services/pluginsManager/pluginsEventBus';
import { searchFitBounds } from '@/services/pluginsManager/map/triggerSearch/searchFitBounds';
import { reactionsFixture } from '@/models/fixtures/reactionFixture';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { apiPath } from '@/redux/apiPath';
import { HttpStatusCode } from 'axios';
import { bioEntityFixture } from '@/models/fixtures/bioEntityFixture';
import { clickHandleReaction } from '@/components/Map/MapViewer/MapViewerVector/listeners/mouseClick/clickHandleReaction';
import { newReactionFixture } from '@/models/fixtures/newReactionFixture';

const mockedAxiosClient = mockNetworkNewAPIResponse();
jest.mock('../../../../../../services/pluginsManager/map/triggerSearch/searchFitBounds');
const eventBusDispatchEventSpy = jest.spyOn(PluginsEventBus, 'dispatchEvent');

describe('clickHandleReaction', () => {
  let dispatch: jest.Mock;
  let modelId = 1;
  let reactionId = 1;
  const hasFitBounds = true;

  beforeEach(() => {
    jest.clearAllMocks();
    dispatch = jest.fn();
  });

  it('dispatches getMultiBioEntityByIds, selectTab, and openBioEntityDrawerById, then triggers PluginsEventBus and searchFitBounds when hasFitBounds is true', async () => {
    dispatch = jest.fn(() => {
      return {
        unwrap: jest.fn().mockResolvedValue([]),
        payload: {
          data: [reactionsFixture[0]],
        },
      };
    });
    reactionId = reactionsFixture[0].id;
    modelId = reactionsFixture[0].modelId;

    mockedAxiosClient
      .onGet(apiPath.getReactionByIdInNewApi(reactionId, modelId))
      .reply(HttpStatusCode.Ok, bioEntityFixture);

    [
      ...reactionsFixture[0].products,
      ...reactionsFixture[0].reactants,
      ...reactionsFixture[0].modifiers,
    ].forEach(element => {
      mockedAxiosClient
        .onGet(
          apiPath.getElementById('aliasId' in element ? element.aliasId : element.element, modelId),
        )
        .reply(HttpStatusCode.Ok, bioEntityFixture);
    });
    clickHandleReaction(dispatch, hasFitBounds)(
      [],
      [{ ...newReactionFixture, id: reactionId }],
      reactionId,
      modelId,
    );
    expect(dispatch).toHaveBeenCalledTimes(5);
    expect(dispatch).toHaveBeenCalledWith(openReactionDrawerById(reactionId));
    expect(dispatch).toHaveBeenCalledWith(selectTab(''));
    expect(eventBusDispatchEventSpy).toHaveBeenCalled();
    expect(searchFitBounds).toHaveBeenCalled();
  });

  it('does not call searchFitBounds when hasFitBounds is false', async () => {
    const mockBioEntities = [{ id: 1, name: 'BioEntity 1' }];
    dispatch.mockImplementation(() => ({
      unwrap: jest.fn().mockResolvedValue(mockBioEntities),
    }));

    clickHandleReaction(dispatch, false)([], [{ ...newReactionFixture, id: 1 }], 1, modelId);

    expect(searchFitBounds).not.toHaveBeenCalled();
  });
});
