/* eslint-disable no-magic-numbers */
import { NewReaction, Reaction, ReactionLine } from '@/types/models';
import ReactionTypeEnum from '@/utils/reaction/ReactionTypeEnum';

type ReactionTypeKey = keyof typeof ReactionTypeEnum;

export default function mapNewReactionToReaction(newReaction: NewReaction): Reaction {
  const lines: Array<ReactionLine> = [];
  let start;
  let end;
  newReaction.line.segments.forEach(segment => {
    start = { x: segment.x1, y: segment.y1 };
    end = { x: segment.x2, y: segment.y2 };
    lines.push({ start, end, type: '' });
  });
  [
    ...newReaction.products,
    ...newReaction.reactants,
    ...newReaction.modifiers,
    ...newReaction.operators,
  ].forEach(element => {
    element.line.segments.forEach(segment => {
      start = { x: segment.x1, y: segment.y1 };
      end = { x: segment.x2, y: segment.y2 };
      lines.push({ start, end, type: '' });
    });
  });

  return {
    centerPoint: { x: 0, y: 0 },
    hierarchyVisibilityLevel: '',
    id: newReaction.id,
    kineticLaw: null,
    lines,
    modelId: newReaction.model,
    modifiers: [],
    name: '',
    notes: newReaction.notes,
    products: [],
    reactants: [],
    reactionId: newReaction.idReaction,
    references: newReaction.references,
    type: ReactionTypeEnum[newReaction.sboTerm as ReactionTypeKey],
  };
}
